/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "iconthememanager.h"
#include "defs.h"

#include <kstandarddirs.h>
#include <kfiledialog.h>
#include <ktar.h>
#include <karchive.h>
#include <kmessagebox.h>
#include <qdir.h>

#include <fstream>
#include <string>
using namespace std;

IconThemeManager::IconThemeManager()
{
	this->findIconThemes();
	m_currentTheme = DEFAULT_ICON_THEME;
}


IconThemeManager::~IconThemeManager()
{
}


void IconThemeManager::findIconThemes()
{
	KStandardDirs ksd;
	QStringList dirs = ksd.findDirs("data", "kosd/themes/");
	m_iconThemes.clear();

	if (!dirs.empty())
	{
		QDir themeDir;
		QStringList::iterator it, it2;
		QStringList entries;

		for (it = dirs.begin(); it != dirs.end(); ++it)
		{
			themeDir.setPath(*it);
			entries = themeDir.entryList("*", QDir::Dirs, QDir::Name);
			for (it2 = entries.begin(); it2 != entries.end(); it2++)
			{
				if (m_iconThemes.find(*it2) == m_iconThemes.end()) // Avoid duplicates
					m_iconThemes.push_back(*it2);
			}
		}

		// Remove "." and ".." directories
		m_iconThemes.remove(".");
		m_iconThemes.remove("..");
		m_iconThemes.sort();
	}

	if (m_iconThemes.find(m_currentTheme) == m_iconThemes.end())
		m_currentTheme = DEFAULT_ICON_THEME;
}


QStringList IconThemeManager::iconThemeList() const
{
	return m_iconThemes;
}


bool IconThemeManager::setIconTheme(QString theme)
{
	QStringList::iterator it;
	it = m_iconThemes.find(theme);

	if (it == m_iconThemes.end())
	{
		m_currentTheme = DEFAULT_ICON_THEME;
		return false;
	}

	m_currentTheme = *it;
	return true;
}


QString IconThemeManager::iconThemeName() const
{
	return m_currentTheme;
}


QString IconThemeManager::author(QString theme) const
{
	if (m_iconThemes.find(theme) == m_iconThemes.end())
		return QString("Anonymous");

	ifstream inFile;
	QString fileName = locate("appdata", "themes/" + theme + "/author");
	if (!QFile::exists(fileName))
		return QString("Anonymous");

	inFile.open(fileName, ios_base::in);

	QString author;
	string name;
	while (inFile >> name)
		author = author + " " + name;

	inFile.close();
	return author;
}


QString IconThemeManager::author() const
{
	return this->author(m_currentTheme);
}


QImage IconThemeManager::icon(QString theme, QString name) const
{
	if (m_iconThemes.find(theme) == m_iconThemes.end())
		theme = DEFAULT_ICON_THEME;

	// Try to find the requested icon in the given theme
	QString fileName = "themes/" + theme + "/" + name;
	QString imagePath = locate("appdata", fileName);

	// If the icon isn't found, try to find it in the default theme
	if (!QFile::exists(imagePath))
		imagePath = locate("appdata", "themes/" + QString(DEFAULT_ICON_THEME) + "/" + name);

	// If the icon isn't found there return an empty image
	if (!QFile::exists(imagePath))
		return QImage();

	return QImage(imagePath);
}


QImage IconThemeManager::icon(QString name) const
{
	return this->icon(m_currentTheme, name);
}


void IconThemeManager::installNewIconTheme()
{
	QString filePath = KFileDialog::getOpenFileName("", "*.tar.gz *.tar.bz2", 0, "Select an icon theme");
	if (filePath != QString::null)
	{
		if (QFile::exists(filePath))
		{
			KTar tarball(filePath);
			tarball.open(IO_ReadOnly);
			const KArchiveDirectory *dir = tarball.directory();
			QStringList entries = dir->entries();

			if (entries.size() == 1)
			{
				const KArchiveEntry *entry = dir->entry(entries[0] + "/author");
				if (entry)
				{
					QString dest = locateLocal("appdata", "themes/", true);
					dir->copyTo(dest, true);
					tarball.close();

					// Update theme list
					this->findIconThemes();
					return;
				}
			}

			// If we reached this point, the archive is invalid
			KMessageBox::error(0, "This archive doesn't contain a valid icon theme.", "Invalid archive!");
			tarball.close();
			return;
		}
	}
}

