/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kosd.h"
#include "kosdglobal.h"
#include "defs.h"

#include <qlabel.h>

#include <klocale.h>
#include <kglobalaccel.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kkeydialog.h>

#include <fstream>
#include <iostream>
#include <cstdlib>
using namespace std;

KOSD::KOSD()
	: KUniqueApplication("KOSD")
{
	m_mixer = KOSDGlobal::mixer();
	m_osd = new OSD();
	m_configWindow = 0;
	m_aboutWindow = new KAboutApplication(0, "aboutWindow", true);

	this->initializeConfiguration();
	this->slotUpdateConfiguration();

	m_globalAccel = new KGlobalAccel(this);
	m_globalAccel->insert("RaiseVolume", "Raise Volume", QString::null, KShortcut("XF86AudioRaiseVolume"),
		KShortcut(), this, SLOT(slotRaiseVolume()));
	m_globalAccel->insert("LowerVolume", "Lower Volume", QString::null, KShortcut("XF86AudioLowerVolume"),
		KShortcut(), this, SLOT(slotLowerVolume()));
	m_globalAccel->insert("MuteVolume", "Mute Volume", QString::null, KShortcut("XF86AudioMute"),
		KShortcut(), this, SLOT(slotMuteVolume()));
	m_globalAccel->insert("ShowVolume", "Show Volume", QString::null, KShortcut(),
		KShortcut(), this, SLOT(slotShowVolume()));
	m_globalAccel->insert("ShowBrightness", "Show Brightness", QString::null, KShortcut(),
		KShortcut(), this, SLOT(slotShowBrightness()));
	m_globalAccel->insert("ShowBrightnessUp", "Show Brightness Up", QString::null, KShortcut(),
		KShortcut(), this, SLOT(slotShowBrightnessUp()));
	m_globalAccel->insert("ShowBrightnessDown", "Show Brightness Down", QString::null, KShortcut(),
		KShortcut(), this, SLOT(slotShowBrightnessDown()));
	m_globalAccel->insert("ShowBattery", "Show Battery Charge", QString::null, KShortcut(),
		KShortcut(), this, SLOT(slotShowBattery()));

	m_globalAccel->readSettings();
	m_globalAccel->updateConnections();
}


KOSD::~KOSD()
{
	delete m_osd;
	delete m_aboutWindow;
	delete m_globalAccel;
}


void KOSD::initializeConfiguration()
{
	// Ensure that all entries have values
	KConfig *config = KGlobal::config();

	config->setGroup("Appearance");
	if (config->readNumEntry("HPos", -1) == -1)
		config->writeEntry("HPos", OSD_DEFAULT_HPOS);
	if (config->readNumEntry("VPos", -1) == -1)
		config->writeEntry("VPos", OSD_DEFAULT_VPOS);

	if (config->readNumEntry("Size", -1) == -1)
		config->writeEntry("Size", OSD_DEFAULT_SIZE);

	if (!config->readColorEntry("BackColor").isValid())
		config->writeEntry("BackColor", OSD_DEFAULT_BACK_COLOR);
	if (!config->readColorEntry("FrontColor").isValid())
		config->writeEntry("FrontColor", OSD_DEFAULT_FRONT_COLOR);

	if (!config->hasKey("DrawTranslucent"))
		config->writeEntry("DrawTranslucent", true);
	if (config->readNumEntry("Opacity", -1) == -1)
		config->writeEntry("Opacity", OSD_DEFAULT_OPACITY);

	if (config->readEntry("IconTheme", "").isEmpty())
		config->writeEntry("IconTheme", DEFAULT_ICON_THEME);

	config->setGroup("Behavior");
	if (config->readNumEntry("Timeout", -1) == -1)
		config->writeEntry("Timeout", OSD_DEFAULT_TIMEOUT);
	if (!config->hasKey("LockPrimaryScreen"))
		config->writeEntry("LockPrimaryScreen", true);

	if (config->readEntry("MixerDevice", "").isEmpty())
		config->writeEntry("MixerDevice", "Unknown");
	if (config->readNumEntry("MixerChannel", -1) == -1)
		config->writeEntry("MixerChannel", 0);
	if (config->readNumEntry("VolumeStep", -1) == -1)
		config->writeEntry("VolumeStep", DEFAULT_VOLUME_STEP);

	if (config->readEntry("MaxBrightnessPath", "").isEmpty())
		config->writeEntry("MaxBrightnessPath", DEFAULT_MAX_BRIGHTNESS_PATH);
	if (config->readEntry("ActualBrightnessPath", "").isEmpty())
		config->writeEntry("ActualBrightnessPath", DEFAULT_ACTUAL_BRIGHTNESS_PATH);

	if (config->readEntry("ChargeFullPath", "").isEmpty())
		config->writeEntry("ChargeFullPath", DEFAULT_CHARGE_FULL_PATH);
	if (config->readEntry("ChargeNowPath", "").isEmpty())
		config->writeEntry("ChargeNowPath", DEFAULT_CHARGE_NOW_PATH);

	config->sync();
}


int KOSD::newInstance()
{
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	if (args->isSet("configure"))
		this->slotShowConfigWindow();
	else if (args->isSet("about"))
		this->slotShowAboutWindow();
	else if (args->isSet("quit"))
		this->quit();

	args->clear();
	return 0;
}


void KOSD::slotShowAboutWindow()
{
	m_aboutWindow->show();
}


void KOSD::slotShowConfigWindow()
{
	m_configWindow = new ConfigurationDialog(0, "Configuration", Qt::WType_Dialog);

	connect(m_configWindow, SIGNAL(changed()), this, SLOT(slotUpdateConfiguration()));
	connect(m_configWindow, SIGNAL(aboutClicked()), this, SLOT(slotShowAboutWindow()));
	connect(m_configWindow, SIGNAL(shortcutsClicked()), this, SLOT(slotConfigureShortcuts()));
	connect(m_configWindow, SIGNAL(finished()), this, SLOT(slotDeleteConfigWindow()));

	m_configWindow->show();
}


void KOSD::slotDeleteConfigWindow()
{
	m_configWindow->delayedDestruct();
}


void KOSD::slotUpdateConfiguration()
{
	KConfig *config = KGlobal::config();
	config->setGroup("Behavior");

	m_pathToActualBrightness = config->readEntry("ActualBrightnessPath");
	m_pathToMaxBrightness = config->readEntry("MaxBrightnessPath");

	m_pathToChargeNow = config->readEntry("ChargeNowPath");
	m_pathToChargeFull = config->readEntry("ChargeFullPath");

	config->setGroup("Appearance");
	KOSDGlobal::iconThemeManager()->setIconTheme(config->readEntry("IconTheme"));

	m_osd->updateConfiguration();
	m_mixer->updateConfiguration();
}


void KOSD::slotRaiseVolume()
{
	m_mixer->setMute(false);
	unsigned int volume = m_mixer->volumeUp();
	m_osd->showVolume(volume, VolumeUp);
}


void KOSD::slotLowerVolume()
{
	m_mixer->setMute(false);
	unsigned int volume = m_mixer->volumeDown();
	m_osd->showVolume(volume, VolumeDown);
}


void KOSD::slotMuteVolume()
{
	bool mute = m_mixer->toggleMute();
	unsigned int volume = m_mixer->volume();

	if (mute)
		m_osd->showVolume(volume, VolumeMute);
	else
		m_osd->showVolume(volume, VolumeUnmute);

}


void KOSD::slotShowVolume()
{
	bool mute = m_mixer->isMuted();
	unsigned int volume = m_mixer->volume();

	if (mute)
		m_osd->showVolume(volume, VolumeMute);
	else
		m_osd->showVolume(volume, VolumeShow);
}


void KOSD::showBrightnessHelper(OSDType type)
{
	ifstream inFile;

	inFile.open(m_pathToActualBrightness, ios_base::in);
	int actualBrightness;
	inFile >> actualBrightness;
	inFile.close();

	inFile.open(m_pathToMaxBrightness, ios_base::in);
	int maxBrightness;
	inFile >> maxBrightness;
	inFile.close();
	
	int brightness = (int)(100.0f / maxBrightness * actualBrightness);
	m_osd->showBrightness(brightness, type);
}


void KOSD::slotShowBrightness()
{
	this->showBrightnessHelper(BrightnessShow);
}


void KOSD::slotShowBrightnessUp()
{
	this->showBrightnessHelper(BrightnessUp);
}


void KOSD::slotShowBrightnessDown()
{
	this->showBrightnessHelper(BrightnessDown);
}


void KOSD::slotShowBattery()
{
	ifstream inFile;

	inFile.open(m_pathToChargeNow, ios_base::in);
	int chargeNow;
	inFile >> chargeNow;
	inFile.close();

	inFile.open(m_pathToChargeFull, ios_base::in);
	int chargeFull;
	inFile >> chargeFull;
	inFile.close();
	
	int charge;
	if (chargeFull <= chargeNow) // Avoid rounding error
		charge = 100;
	else
		charge = (int)(100.0f / chargeFull * chargeNow);
	m_osd->showBattery(charge);
}


void KOSD::slotConfigureShortcuts()
{
	KKeyDialog::configure(m_globalAccel, 0, false);
	m_globalAccel->writeSettings();
	m_globalAccel->updateConnections();
}


void KOSD::quit()
{
	exit(0);
}


// DCOP Functions
void KOSD::showVolume(unsigned int percent)
{
	m_osd->showVolume(percent, VolumeShow);
}


void KOSD::showVolumeUp(unsigned int percent)
{
	m_osd->showVolume(percent, VolumeUp);
}


void KOSD::showVolumeDown(unsigned int percent)
{
	m_osd->showVolume(percent, VolumeDown);
}


void KOSD::showVolumeUnmute(unsigned int percent)
{
	m_osd->showVolume(percent, VolumeUnmute);
}


void KOSD::showVolumeMute()
{
	m_osd->showVolume(0, VolumeMute);
}


void KOSD::showBrightness(unsigned int percent)
{
	m_osd->showBrightness(percent, BrightnessShow);
}


void KOSD::showBrightnessUp(unsigned int percent)
{
	m_osd->showBrightness(percent, BrightnessUp);
}


void KOSD::showBrightnessDown(unsigned int percent)
{
	m_osd->showBrightness(percent, BrightnessDown);
}


void KOSD::showBattery(unsigned int percent)
{
	m_osd->showBattery(percent);
}


void KOSD::showVolume()
{
	this->slotShowVolume();
}


void KOSD::raiseVolume()
{
	this->slotRaiseVolume();
}


void KOSD::lowerVolume()
{
	this->slotLowerVolume();
}


void KOSD::muteVolume()
{
	this->slotMuteVolume();
}


void KOSD::showBrightness()
{
	this->slotShowBrightness();
}


void KOSD::showBrightnessUp()
{
	this->slotShowBrightnessUp();
}


void KOSD::showBrightnessDown()
{
	this->slotShowBrightnessDown();
}


void KOSD::showBattery()
{
	this->slotShowBattery();
}


void KOSD::showCustom(QString iconPath, QString text, unsigned int percent)
{
	m_osd->showCustom(iconPath, text, percent);
}

#include "kosd.moc"
