/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mixer.h"

#include <kuniqueapplication.h>
#include <kglobal.h>
#include <kconfig.h>
#include <dcopclient.h>

#include <qvaluelist.h>

#include <iostream>
using namespace std;

Mixer::Mixer()
{
	m_dcopClient = KUniqueApplication::dcopClient();
	m_volumeStep = 5;

	this->refreshMixerDeviceList();
	if (m_mixersAvailable)
		m_currentMixerDevice = &m_mixerDevices.at(0);

	m_mixerChannel = 0;
}


Mixer::~Mixer()
{
	delete m_dcopClient;
}


bool Mixer::initialize()
{
	this->refreshMixerDeviceList();
	if (m_mixersAvailable)
		this->updateConfiguration();

	return m_mixersAvailable;
}


void Mixer::refreshMixerDeviceList()
{
	QByteArray data, replyData;
	QCString replyType;
	QString name;

	m_mixerDevices.clear();

	QCStringList objects = m_dcopClient->remoteObjects("kmix");
	for (QCStringList::iterator it = objects.begin(); it != objects.end(); it++)
	{
		QCString identifier = *it;
		QCString index = identifier;
		if (identifier.contains("Mixer"))
		{
			MixerDevice md;
			md.identifier = identifier;
			unsigned int id = index.remove(0, 5).toUInt();
			md.index = id;

			if (!m_dcopClient->call("kmix", identifier, "mixerName()", data, replyType, replyData))
			{
				cout << "Couldn't retrieve name for mixer device " << identifier << " from KMix!" << endl;
				continue;
			}

			QDataStream reply(replyData, IO_ReadOnly);
			reply >> name;
			md.name = name;

			// Find available channels (limit to 30 channels max)
			int numChannels = 0;
			bool available = false;
			for (int i = 0; i < 30; i++)
			{
				QDataStream arg(data, IO_WriteOnly);
				arg << i;
				if (!m_dcopClient->call("kmix", identifier, "isAvailableDevice(int)", data, replyType, replyData))
				{
					cout << "Couldn't retrieve available channels for " << identifier << " from KMix!" << endl;
					continue;
				}
				QDataStream reply(replyData, IO_ReadOnly);
				reply >> available;
				if (available)
					numChannels++;
				else
					continue;
			}
			md.channels = numChannels;

			m_mixerDevices.push_back(md);
		}
	}

	m_mixersAvailable = m_mixerDevices.empty() ? false : true;
}


void Mixer::updateConfiguration()
{
	KConfig *config = KGlobal::config();
	config->setGroup("Behavior");

	if (this->mixerDeviceAvailable(config->readEntry("MixerDevice")))
		this->setMixerDevice(config->readEntry("MixerDevice"));
	else
		this->setMixerDevice(0);
	this->setMixerChannel(config->readNumEntry("MixerChannel"));
	m_volumeStep = config->readNumEntry("VolumeStep");
}


QStringList Mixer::mixerDeviceList()
{
	if (!m_mixersAvailable)
		this->initialize();

	QStringList mdl;
	vector<MixerDevice>::const_iterator it;
	for (it = m_mixerDevices.begin(); it != m_mixerDevices.end(); it++)
		mdl.push_back(it->name);

	return mdl;
}


bool Mixer::mixerDeviceAvailable(unsigned int index) const
{
	vector<MixerDevice>::const_iterator it;
	for (it = m_mixerDevices.begin(); it != m_mixerDevices.end(); it++)
	{
		if (it->index == index)
			return true;
	}

	return false;
}


bool Mixer::mixerDeviceAvailable(QString name) const
{
	vector<MixerDevice>::const_iterator it;
	for (it = m_mixerDevices.begin(); it != m_mixerDevices.end(); it++)
	{
		if (it->name == name)
			return true;
	}

	return false;
}


bool Mixer::setMixerDevice(unsigned int index)
{
	if (!m_mixersAvailable)
		this->initialize();

	if (this->mixerDeviceAvailable(index))
	{
		m_currentMixerDevice = &m_mixerDevices.at(index);
		return true;
	}

	return false;
}


bool Mixer::setMixerDevice(QString name)
{
	if (!m_mixersAvailable)
		this->initialize();

	if (this->mixerDeviceAvailable(name))
	{
		vector<MixerDevice>::const_iterator it;
		for (it = m_mixerDevices.begin(); it != m_mixerDevices.end(); it++)
		{
			if (it->name == name)
			{
				m_currentMixerDevice = &m_mixerDevices.at(it->index);
				return true;
			}
		}
	}

	return false;
}


bool Mixer::setMixerChannel(unsigned int channel)
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable || (channel > m_currentMixerDevice->channels-1))
	{
		cout << "Error: Couldn't set mixer channel!" << endl;
		return false;
	}

	m_mixerChannel = channel;
	return true;
}


QString Mixer::mixerDeviceName()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
		return QString("");

	return m_currentMixerDevice->name;
}


unsigned int Mixer::mixerDeviceChannels()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
		return 0;

	return m_currentMixerDevice->channels;
}


unsigned int Mixer::volume()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
	{
		cout << "Error: No mixers available!" << endl;
		return 0;
	}

	QByteArray data, replyData;
	QCString replyType;
	QDataStream arg(data, IO_WriteOnly);
	arg << m_mixerChannel;
	unsigned int volume;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "volume(int)", data, replyType, replyData))
	{
		cout << "Error: Couldn't retrieve volume from KMix!" << endl;
		return 0;
	}

	QDataStream reply(replyData, IO_ReadOnly);
	reply >> volume;
	return volume;
}


bool Mixer::isMuted()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
	{
		cout << "Error: No mixers available!" << endl;
		return true;
	}

	QByteArray data, replyData;
	QCString replyType;
	QDataStream arg(data, IO_WriteOnly);
	arg << m_mixerChannel;
	bool mute;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "mute(int)", data, replyType, replyData))
	{
		cout << "Error: Couldn't retrieve state from KMix!" << endl;
		return true;
	}

	QDataStream reply(replyData, IO_ReadOnly);
	reply >> mute;
	return mute;
}


unsigned int Mixer::volumeUp()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
	{
		cout << "Error: No mixers available!" << endl;
		return 0;
	}

	QByteArray data, replyData;
	QCString replyType;
	QDataStream arg(data, IO_WriteOnly);
	arg << m_mixerChannel;
	int volume;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "volume(int)", data, replyType, replyData))
	{
		cout << "Error: Couldn't retrieve volume from KMix!" << endl;
		return 0;
	}

	QDataStream reply(replyData, IO_ReadOnly);
	reply >> volume;

	volume += m_volumeStep;
	if (volume > 100)
		volume = 100;

	arg << (volume + 1); // A bug in KMix causes it to always set the volume to (givenVolume - 1)
	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "setVolume(int, int)", data, replyType, replyData))
		cout << "Error: Couldn't set volume!" << endl;

	return volume;
}


unsigned int Mixer::volumeDown()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
	{
		cout << "Error: No mixers available!" << endl;
		return 0;
	}

	QByteArray data, replyData;
	QCString replyType;
	QDataStream arg(data, IO_WriteOnly);
	arg << m_mixerChannel;
	int volume;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "volume(int)", data, replyType, replyData))
	{
		cout << "Error: Couldn't retrieve volume from KMix!" << endl;
		return 0;
	}

	QDataStream reply(replyData, IO_ReadOnly);
	reply >> volume;

	volume -= m_volumeStep;
	if (volume < 0)
		volume = 0;

	arg << (volume + 1); // A bug in KMix causes it to always set the volume to (givenVolume - 1)
	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "setVolume(int, int)", data, replyType, replyData))
		cout << "Error: Couldn't set volume!" << endl;


	return volume;
}


bool Mixer::toggleMute()
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
	{
		cout << "Error: No mixers available!" << endl;
		return true;
	}

	QByteArray data, replyData;
	QCString replyType;
	QDataStream arg(data, IO_WriteOnly);
	arg << m_mixerChannel;
	bool mute;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "toggleMute(int)", data, replyType, replyData))
		cout << "Error: Couldn't toggle mute!" << endl;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "mute(int)", data, replyType, replyData))
	{
		cout << "Error: Couldn't retrieve state from KMix!" << endl;
		return true;
	}

	QDataStream reply(replyData, IO_ReadOnly);
	reply >> mute;
	return mute;
}


void Mixer::setMute(bool mute)
{
	if (!m_mixersAvailable)
		this->initialize();

	if (!m_mixersAvailable)
	{
		cout << "Error: No mixers available!" << endl;
		return;
	}

	QByteArray data, replyData;
	QCString replyType;
	QDataStream arg(data, IO_WriteOnly);
	arg << m_mixerChannel;
	arg << mute;

	if (!m_dcopClient->call("kmix", m_currentMixerDevice->identifier, "setMute(int, bool)", data, replyType, replyData))
		cout << "Error: Couldn't set mute!" << endl;
}
