/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "osd.h"
#include "kosdglobal.h"
#include "defs.h"
#include "iconthememanager.h"

#include <kglobal.h>
#include <kglobalsettings.h>
#include <kconfig.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qbitmap.h>
#include <qfile.h>
#include <qcursor.h>

OSD::OSD()
	: QWidget(0, "osd", Qt::WType_TopLevel | Qt::WNoAutoErase | Qt::WStyle_Customize | Qt::WX11BypassWM | Qt::WStyle_StaysOnTop)
{
	this->setBackgroundMode(QWidget::NoBackground);
	this->setFocusPolicy(QWidget::NoFocus);

	m_volumeImage = new QImage();
	m_volumeUpImage = new QImage();
	m_volumeDownImage = new QImage();
	m_volumeMuteImage = new QImage();

	m_brightnessImage = new QImage();
	m_brightnessUpImage = new QImage();
	m_brightnessDownImage = new QImage();

	m_batteryImage = new QImage();
	m_customImage = new QImage();

	m_valueBar = new KProgress(this, "ValueBar");
	m_painter = new QPainter();

	m_backColor = new QColor();
	m_frontColor = new QColor();

	m_textFont = KGlobalSettings::generalFont();

	m_desktop = QApplication::desktop();

	m_timer = new QTimer(this, "Timer");
	connect(m_timer, SIGNAL(timeout()), this, SLOT(slotHide()));

	m_isShown = false;
	m_screenshot = new QPixmap();

	this->updateConfiguration();
}


OSD::~OSD()
{
	delete m_volumeImage;
	delete m_volumeUpImage;
	delete m_volumeDownImage;
	delete m_volumeMuteImage;
	delete m_brightnessImage;
	delete m_brightnessUpImage;
	delete m_brightnessDownImage;
	delete m_batteryImage;
	delete m_customImage;
	delete m_painter;
	delete m_backColor;
	delete m_frontColor;
	delete m_valueBar;
	delete m_timer;
	delete m_screenshot;
}


void OSD::makeNewScreenshot()
{
	*m_screenshot = QPixmap::grabWindow(qt_xrootwin(), m_xPos, m_yPos, m_width, m_height);
}


void OSD::updatePosition()
{
	QRect rect;
	if (m_lockPrimaryScreen)
		rect = m_desktop->screenGeometry(m_desktop->primaryScreen());
	else
		rect = m_desktop->screenGeometry(QCursor::pos());

	m_xPos = rect.x() + (unsigned int)((rect.width() - m_width) / 100.0f * m_hPos);
	m_yPos = rect.y() + (unsigned int)((rect.height() - m_height) / 100.0f * m_vPos);
	this->setGeometry(m_xPos, m_yPos, m_width, m_height);
}


void OSD::updateConfiguration()
{
	KConfig *config = KGlobal::config();
	config->setGroup("Appearance");
	m_scaleFactor = OSD_MIN_SCALE_FACTOR + ((1.0f - OSD_MIN_SCALE_FACTOR) * (config->readNumEntry("Size") / 100.0f));

	m_width = (unsigned int)(OSD_BASE_WIDTH * m_scaleFactor);
	m_height = (unsigned int)(OSD_BASE_HEIGHT * m_scaleFactor);

	m_hPos = config->readNumEntry("HPos");
	m_vPos = config->readNumEntry("VPos");

	*m_backColor = config->readColorEntry("BackColor");
	*m_frontColor = config->readColorEntry("FrontColor");

	unsigned int imageX = (unsigned int)(50.0f * m_scaleFactor);
	unsigned int imageY = (unsigned int)(30.0f * m_scaleFactor);
	m_imageSize = (unsigned int)(128.0f * m_scaleFactor);
	m_imageRect.setRect(imageX, imageY, m_imageSize, m_imageSize);

	IconThemeManager *itm = KOSDGlobal::iconThemeManager();
	*m_volumeImage = itm->icon("volume.png").smoothScale(m_imageSize, m_imageSize);
	*m_volumeMuteImage = itm->icon("volume_mute.png").smoothScale(m_imageSize, m_imageSize);
	*m_volumeUpImage = itm->icon("volume_up.png").smoothScale(m_imageSize, m_imageSize);
	*m_volumeDownImage = itm->icon("volume_down.png").smoothScale(m_imageSize, m_imageSize);

	*m_brightnessImage = itm->icon("brightness.png").smoothScale(m_imageSize, m_imageSize);
	*m_brightnessUpImage = itm->icon("brightness_up.png").smoothScale(m_imageSize, m_imageSize);
	*m_brightnessDownImage = itm->icon("brightness_down.png").smoothScale(m_imageSize, m_imageSize);

	*m_batteryImage = itm->icon("battery.png").smoothScale(m_imageSize, m_imageSize);

	unsigned int textX = (unsigned int)(180.0f * m_scaleFactor);
	unsigned int textY = imageY;
	unsigned int textWidth = (unsigned int)(380.0f * m_scaleFactor);
	unsigned int textHeight = m_imageSize;
	m_textRect.setRect(textX, textY, textWidth, textHeight);
	m_textFont.setPixelSize((int)(m_height / 100.0f * 17.0f));
	m_textFont.setBold(true);

	unsigned int valueX = (unsigned int)(50.0f * m_scaleFactor);
	unsigned int valueY = (unsigned int)(175.0f * m_scaleFactor);
	unsigned int valueWidth = (unsigned int)(500.0f * m_scaleFactor);
	unsigned int valueHeight = (unsigned int)(47.0f * m_scaleFactor);
	m_valueBar->setGeometry(valueX, valueY, valueWidth, valueHeight);
	m_valueBar->setPaletteBackgroundColor(*m_backColor);

	QFont font;
	font.setPixelSize((int)(valueHeight / 2.0f));
	m_valueBar->setFont(font);

	m_isTranslucent = config->readBoolEntry("DrawTranslucent");
	m_opacity = config->readNumEntry("Opacity");

	config->setGroup("Behavior");
	m_timeout = config->readNumEntry("Timeout");
	m_lockPrimaryScreen = config->readNumEntry("LockPrimaryScreen");
}


void OSD::showVolume(unsigned int volume, OSDType type)
{
	this->updatePosition();

	if (volume > 100)
		volume = 100;

	if (m_isTranslucent && !m_isShown)
		this->makeNewScreenshot();

	this->QWidget::show();
	this->QWidget::update();
	m_isShown = true;

	if (type == VolumeMute)
		m_valueBar->setValue(0);
	else
		m_valueBar->setValue(volume);

	switch (type)
	{
		case VolumeUp:
			m_currentImage = m_volumeUpImage;
			m_text = "Volume Up";
			break;
		case VolumeDown:
			m_currentImage = m_volumeDownImage;
			m_text = "Volume Down";
			break;
		case VolumeShow:
			m_currentImage = m_volumeImage;
			m_text = "Volume";
			break;
		case VolumeMute:
			m_currentImage = m_volumeMuteImage;
			m_text = "Mute";
			break;
		case VolumeUnmute:
			m_currentImage = m_volumeImage;
			m_text = "Unmute";
			break;
		default:
			m_text = "Error!";
			break;	
	}

	m_timer->start(m_timeout, true);
}


void OSD::showBrightness(unsigned int brightness, OSDType type)
{
	this->updatePosition();

	if (brightness > 100)
		brightness = 100;

	if (m_isTranslucent && !m_isShown)
		this->makeNewScreenshot();

	this->QWidget::show();
	this->QWidget::update();
	m_isShown = true;

	m_valueBar->setValue(brightness);

	switch (type)
	{
		case BrightnessUp:
			m_currentImage = m_brightnessUpImage;
			m_text = "Brightness Up";
			break;
		case BrightnessDown:
			m_currentImage = m_brightnessDownImage;
			m_text = "Brightness Down";
			break;
		case BrightnessShow:
			m_currentImage = m_brightnessImage;
			m_text = "Brightness";
			break;
		default:
			m_text = "Error";
			break;
	}

	m_timer->start(m_timeout, true);
}


void OSD::showBattery(unsigned int charge)
{
	this->updatePosition();

	if (charge > 100)
		charge = 100;

	if (m_isTranslucent && !m_isShown)
		this->makeNewScreenshot();

	this->QWidget::show();
	this->QWidget::update();
	m_isShown = true;

	m_currentImage = m_batteryImage;
	m_text = "Battery charge";
	m_valueBar->setValue(charge);

	m_timer->start(m_timeout, true);
}


void OSD::showCustom(QString iconPath, QString text, unsigned int percent)
{
	this->updatePosition();

	if (percent > 100)
		percent = 100;

	if (m_isTranslucent && !m_isShown)
		this->makeNewScreenshot();

	this->QWidget::show();
	this->QWidget::update();
	m_isShown = true;

	if (QFile::exists(iconPath))
		*m_customImage = QImage(iconPath).smoothScale(m_imageSize, m_imageSize);
	else
		*m_customImage = QImage();

	m_currentImage = m_customImage;

	m_text = text;
	m_valueBar->setValue(percent);

	m_timer->start(m_timeout, true);
}


void OSD::slotHide()
{
	this->hide();
	m_isShown = false;
}


void OSD::paintEvent(QPaintEvent*)
{
	m_painter->begin(this);

	static QBitmap mask;
	mask.resize(m_width, m_height);
	mask.fill(Qt::black);

	QPainter p(&mask);
	p.setBrush(Qt::white);
	p.drawRoundRect(0, 0, m_width, m_height, 18, 40);
	this->setMask(mask);

	if (m_isTranslucent)
	{
		KPixmap background(*m_screenshot);
		KPixmapEffect::fade(background, m_opacity/100.0f, *m_backColor);
		m_painter->drawPixmap(0, 0, background);
	}
	else
	{
		m_painter->setPen(QPainter::NoPen);
		m_painter->setBrush(*m_backColor);
		m_painter->drawRoundRect(0, 0, m_width, m_height, 18, 40);
	}

	m_painter->setPen(*m_frontColor);
	m_painter->setBrush(QPainter::NoBrush);
	m_painter->drawRoundRect(0, 0, m_width, m_height, 18, 40);

	m_painter->setFont(m_textFont);
	m_painter->drawText(m_textRect, Qt::AlignCenter | Qt::WordBreak, m_text);

	m_painter->drawImage(m_imageRect, *m_currentImage);
	m_valueBar->show();

	m_painter->end();
}


#include "osd.moc"
